import { system, world, EntityEquippableComponent, EquipmentSlot, ItemStack} from "@minecraft/server";

world.beforeEvents.itemUseOn.subscribe(use =>{
	try{
		const p = use.source;
		const item = use.itemStack;
		const block = use.block;
		const l = [block.location.x, block.location.y, block.location.z]
		const damage = item.getComponent("durability").damage;
		const maxDurability = item.getComponent("durability").maxDurability;
		if(item.typeId == "new:wyvern_bone_hoe"){
			if(block.typeId.includes("grass") || block.typeId.includes("dirt")){
				p.runCommandAsync(`fill ${l[0]} ${l[1]} ${l[2]} ${l[0]} ${l[1]} ${l[2]} farmland`)
			}
		}
		if(item.typeId == "new:skeleton_wyvern_hoe"){
			if(block.typeId.includes("grass") || block.typeId.includes("dirt")){
				p.runCommandAsync(`fill ${l[0]} ${l[1]} ${l[2]} ${l[0]} ${l[1]} ${l[2]} farmland`)
			}
		}
		if(item.typeId == "new:wyvern_bone_shovel"){
			if(block.typeId.includes("grass") || block.typeId.includes("dirt")){
				p.runCommandAsync(`fill ${l[0]} ${l[1]} ${l[2]} ${l[0]} ${l[1]} ${l[2]} grass_path`)
			}
		}
		let axe = ["acacia", "oak", "spruce", "jungle", "dark_oak", "birch", "mangrove", "cherry"]
		for(const ids of axe){
			if(item.typeId == "new:wyvern_bone_axe"){
				if(block.typeId.includes(`${ids}_log`)){
					p.runCommandAsync(`fill ${l[0]} ${l[1]} ${l[2]} ${l[0]} ${l[1]} ${l[2]} stripped_${ids}_log [] replace ${ids}_log`)
				}
			}
		}

	}catch(e){
	}
});

export function getScore(player, objective) {
	return world.scoreboard.getObjective(objective).getScore(player) ?? 0;
}
export function setScore(player, objective, score) {
	return world.scoreboard.getObjective(objective).setScore(player, score) ?? 0;
}
export function addScore(player, objective, score) {
	return world.scoreboard.getObjective(objective).addScore(player, score) ?? 0;
}

const w = world.getDimension("overworld");

world.afterEvents.entitySpawn.subscribe(use =>{
	const e = use.entity;
	if(e.typeId.includes("_wyvern_egg")){
		const nameTag = e.typeId.replace("_wyvern_egg", "")
		e.runCommandAsync("scoreboard objectives add egg_time dummy")
		e.runCommandAsync("scoreboard players set @s egg_time 20")
		e.addTag(`${nameTag}`)
		
	}
});

const wyverns_type = ["fire", "ice", "ground", "poison", "wither", "ender"]
system.runInterval(() =>{
	try{
		for(const p of world.getPlayers()){
			p.runCommandAsync("gamerule commandblockoutput false")
			if(p.hasTag("mount") && p.isJumping){
				p.runCommandAsync("effect @e[family=wyvern_tamed,r=4] levitation 1 5 true")
				p.runCommandAsync("event entity @e[family=wyvern_tamed,r=4] new:wyvern_can_fly")
				p.runCommandAsync("camera @s set minecraft:third_person")
			}
			if(!p.hasTag("mount")){
				p.runCommandAsync("event entity @e[family=wyvern_tamed,r=4] new:wyvern_not_can_fly")
				p.runCommandAsync("camera @s clear")
			}
			for(const types of wyverns_type){
				const wyverns_egg = w.getEntities({type: `new:${types}_wyvern_egg`})
				for(const we of wyverns_egg){
					const egg_time = getScore(we, "egg_time");
					if(!we.hasTag("shaked")){
						if(egg_time == 10){
							we.runCommandAsync("event entity @s[tag=!shaked] new:egg_shaked")
							we.runCommandAsync("tag @s[tag=!shaked] add shaked")
						}
					}
					if(we.hasTag("shaked")){
						if(egg_time == 0){
							we.runCommandAsync("event entity @s[tag=shaked] new:egg_transform")
						}
					}
					const view = p.getEntitiesFromViewDirection({maxDistance: 7});
					view.forEach((look) => {
						let eggTag;
						let firstWord;
						let eggName;
						if(look.entity.typeId == we.typeId){
							look.entity.getTags().forEach(t => {
								if (t.startsWith('new:')) {
									eggTag = `${t.replace("new:", "").split(",")}`
									firstWord = t[4].toUpperCase();
									eggName = `${firstWord}`+`${eggTag.replace(`${t[4]}`, "")}`
									p.runCommandAsync(`title @s actionbar §7--${eggName} Wyvern Egg--\n§6Hatching Time: §b${getScore(look.entity, "egg_time")} min`);
								}
							}); 
						}
					});
				}
				const wyverns = w.getEntities({type: `new:${types}_wyvern`})
				for(const wyvern of wyverns){
					const wInv = wyvern.getComponent("inventory").container;
					for(let i = 0;i<wInv.size;i++){
						const item = wInv.getItem(i)
						if(item){
							if(item.typeId=="new:wyvern_saddle" && i==0 && !wyvern.hasTag("saddled")){
								wyvern.triggerEvent("new:wyvern_saddled")
								wyvern.addTag("saddled")
							}
							if(item.typeId=="minecraft:chest" && i==1 && !wyvern.hasTag("chested")){
								wyvern.triggerEvent("wyvern_chested")
								wyvern.addTag("chested")
							}
							if(item.typeId=="new:wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
								wyvern.triggerEvent("wyvern_claws")
								wyvern.addTag("claws")
							}
							if(item.typeId=="new:gold_wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
								wyvern.triggerEvent("gold_wyvern_claws")
								wyvern.addTag("claws")
							}
							if(item.typeId=="new:iron_wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
								wyvern.triggerEvent("iron_wyvern_claws")
								wyvern.addTag("claws")
							}
							if(item.typeId=="new:diamond_wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
								wyvern.triggerEvent("diamond_wyvern_claws")
								wyvern.addTag("claws")
							}
							if(item.typeId=="new:netherite_wyvern_claws" && i==2 && !wyvern.hasTag("claws")){
								wyvern.triggerEvent("netherite_wyvern_claws")
								wyvern.addTag("claws")
							}
						}
						if(!item && i==0 && wyvern.hasTag("saddled")){
							wyvern.triggerEvent("new:wyvern_unsaddled")
							wyvern.removeTag("saddled")
						}
						if(!item && i==1 && wyvern.hasTag("chested")){
							wyvern.triggerEvent("wyvern_unchested")
							wyvern.removeTag("chested")
						}
						if(!item && i==2 && wyvern.hasTag("claws")){
							wyvern.triggerEvent("wyvern_unclaws")
							wyvern.removeTag("claws")
						}
					}
				}
			}
		}
	}catch(e){

	};
})


system.runInterval(()=>{			
	for(const types of wyverns_type){
		const wyverns_egg = w.getEntities({type: `new:${types}_wyvern_egg`})
		for(const we of wyverns_egg){
			let egg_time = getScore(we, "egg_time");
			if(egg_time > 0){
				we.runCommandAsync(`scoreboard players remove @s egg_time 1`)
			}
		}
	}
}, 1200)

function addEffect(p, effect, duration, level, particles){
	try{
		return p.addEffect(effect, duration*2, {amplifier: level, showParticles: particles}) ?? 0;
	}catch(e){

	}
}


world.afterEvents.itemUse.subscribe(use =>{
	try{
		let itemStack = use.itemStack;
		let p = use.source;
		let damage = itemStack.getComponent("durability").damage;
		let maxDurability = itemStack.getComponent("durability").maxDurability;
		if(itemStack.typeId.includes("_wyvern_wand") && !p.hasTag("cooldown")){
			let arrow = p.dimension.spawnEntity(itemStack.typeId, p.getHeadLocation());
			let projectile = arrow.getComponent('projectile');
			projectile.owner = p;
			projectile.shoot({x: p.getViewDirection().x*3, y: p.getViewDirection().y*3, z: p.getViewDirection().z*3});
			Durability(itemStack, p)
			p.addTag("cooldown")
			system.runTimeout(()=>{
				p.removeTag("cooldown")
			}, 60)
		}
	}catch(e){}
})

world.afterEvents.itemCompleteUse.subscribe(use =>{
	try{
		let p = use.source;
		let item = use.itemStack;
		if(item.typeId == "new:fire_essence"){
			addEffect(p, "fire_resistance", 1800, 4, false)
		}
		if(item.typeId == "new:potion_wyvern_slayer"){
			addEffect(p, "fire_resistance", 3600, 4, false)
			addEffect(p, "strenght", 3600, 4, false)
			addEffect(p, "resistance", 3600, 4, false)
			addEffect(p, "night_vision", 3600, 4, false)
			addEffect(p, "speed", 3600, 1, false)
		}
	}catch(e){

	}
})

world.afterEvents.entityHurt.subscribe(hurt =>{
	try{
		let eHurt = hurt.hurtEntity;
		let damageSource = hurt.damageSource;
		let damage = hurt.damage;
		let cause = damageSource.cause;
		let damaging = damageSource.damagingEntity;
		if(damaging.typeId == "minecraft:player"){
			let equip = damaging.getComponent(EntityEquippableComponent.componentId);
			let hand = equip.getEquipment(EquipmentSlot.Mainhand);
			if(hand){
				if(hand.typeId == "new:poison_wyvern_dagger"){
					addEffect(eHurt, "poison", 40, 2, true)
				}
				if(hand.typeId == "new:ice_wyvern_dagger"){
					addEffect(eHurt, "slowness", 40, 2, true)
				}
				if(hand.typeId == "new:wither_wyvern_dagger"){
					addEffect(eHurt, "wither", 40, 2, true)
				}
			}
		}
	}catch(e){

	};
})


export function Durability(tool, source) {
	try{
		system.run(()=>{
			const component = source.getComponent(EntityEquippableComponent.componentId);
			const durability = tool.getComponent("durability");
			if(tool && durability !== undefined){
				if(durability.damage+2 < durability.maxDurability){
					durability.damage = durability.damage+2;
					component.setEquipment(EquipmentSlot.Mainhand, tool)
				}else if(durability.damage+2 >= durability.maxDurability){
					const air = new ItemStack("minecraft:air")
					component.setEquipment(EquipmentSlot.Mainhand, air)
					source.playSound("random.break")
				}
			}
		})
	}catch(e){}
}